/*
 * Project Name:
     T6963C_240x128 (GLCD Library demo for Toshiba's T6963 Controller)
 * Copyright:
     (c) MikroElektronika, 2018.
 * Revision History:
     20080930:
       - initial release; Credits to Bruno Gavand.
 * Description:
     This code in intended to work with GLCD's based on TOSHIBA T6963C controller.
     Pressing buttons RB0 .. RB5 generates commands for text and graphic displaying.
     This parts may need a -15V power supply on Vee for LCD drive,
     a simple DC/DC converter can be made with a 2N2905, 220 H self, diode & 47 F cap,
     transistor base is driven with PWM through a current limiting resistor.
     This parts have a 8 Kb built-in display RAM, this allows 2 graphics panels
     and one text panel.
 * Test configuration:
     MCU:             PIC18F45K22
                      http://ww1.microchip.com/downloads/en/DeviceDoc/40001412G.pdf
     Dev.Board:       EasyPIC7
                      http://www.mikroe.com/easypic/
     Oscillator:      HS-PLL, 32.0000 MHz
     Ext. Modules:    ac:GLCD_240x128_Adapter_Board on PORTC(Control) and PORTD(Data),
     SW:              mikroC PRO for PIC
                      http://www.mikroe.com/mikroc/pic/
 * NOTES:
     - Put button jumper (J17) into Vcc position and pull-down PORTB.
 */

#include        "__T6963C.h"

// T6963C module connections
char T6963C_dataPort at PORTD;                   // DATA port

sbit T6963C_ctrlwr  at LATC2_bit;                // WR write signal
sbit T6963C_ctrlrd  at LATC1_bit;                // RD read signal
sbit T6963C_ctrlcd  at LATC0_bit;                // CD command/data signal
sbit T6963C_ctrlrst at LATC4_bit;                // RST reset signal
sbit T6963C_ctrlwr_Direction  at TRISC2_bit;     // WR write signal
sbit T6963C_ctrlrd_Direction  at TRISC1_bit;     // RD read signal
sbit T6963C_ctrlcd_Direction  at TRISC0_bit;     // CD command/data signal
sbit T6963C_ctrlrst_Direction at TRISC4_bit;     // RST reset signal

// Signals not used by library, they are set in main function
sbit T6963C_ctrlce at LATC3_bit;                 // CE signal
sbit T6963C_ctrlfs at LATC6_bit;                 // FS signal
sbit T6963C_ctrlmd at LATC5_bit;                 // MD signal
sbit T6963C_ctrlce_Direction  at TRISC3_bit;     // CE signal direction
sbit T6963C_ctrlfs_Direction  at TRISC6_bit;     // FS signal direction
sbit T6963C_ctrlmd_Direction  at TRISC5_bit;     // MD signal direction
// End T6963C module connections

/*
 * bitmap pictures stored in ROM
 */
const code char mikroE_240x128_bmp[];
const code char einstein[];


void main() {

  char txt1[] = " EINSTEIN WOULD HAVE LIKED mE";
  char txt[] =  " GLCD LIBRARY DEMO, WELCOME !";
  char txt2[] = " Partial image demo!";

  unsigned char  panel;         // Current panel
  unsigned int   i;             // General purpose register
  unsigned char  curs;          // Cursor visibility
  unsigned int   cposx, cposy;  // Cursor x-y position

  ANSELB = 0;                   // Configure AN pins as digital
  ANSELC = 0;                   // Disable comparators
  ANSELD = 0;

  TRISB0_bit = 1;               // Set RB0 as input
  TRISB1_bit = 1;               // Set RB1 as input
  TRISB2_bit = 1;               // Set RB2 as input
  TRISB3_bit = 1;               // Set RB3 as input
  TRISB4_bit = 1;               // Set RB4 as input
  TRISB5_bit = 1;               // Set RB5 as input

  T6963C_ctrlce_Direction = 0;
  T6963C_ctrlce = 0;            // Enable T6963C
  T6963C_ctrlfs_Direction = 0;
  T6963C_ctrlfs = 0;            // Font Select 8x8
  T6963C_ctrlmd_Direction = 0;
  T6963C_ctrlmd = 0;            // Column number select

  panel = 0;
  i = 0;
  curs = 0;
  cposx = cposy = 0;

  // Initialize T6963C
  T6963C_init(240, 128, 8);

  /*
   * Enable both graphics and text display at the same time
   */
  T6963C_Graphics(1);
  T6963C_Text(1);

  /*
   * Text messages
   */
  T6963C_write_text(txt, 0, 0, T6963C_ROM_MODE_XOR);
  T6963C_write_text(txt1, 0, 15, T6963C_ROM_MODE_XOR);

  /*
   * Cursor
   */
  T6963C_Cursor_Height(8);       // 8 pixel height
  T6963C_set_cursor(0, 0);       // Move cursor to top left
  T6963C_Cursor(0);              // Cursor off

  /*
   * Draw solid boxes
   */
  T6963C_box(0, 0, 239, 8, T6963C_WHITE);
  T6963C_box(0, 119, 239, 127, T6963C_WHITE);

  /*
   * Draw rectangles
   */
  T6963C_rectangle(0, 0, 239, 127, T6963C_WHITE);
  T6963C_rectangle(20, 20, 219, 107, T6963C_WHITE);
  T6963C_rectangle(40, 40, 199, 87, T6963C_WHITE);
  T6963C_rectangle(60, 60, 179, 67, T6963C_WHITE);

  /*
   * Draw rounded edge rectangle
   */
  T6963C_Rectangle_Round_Edges(10, 10, 229, 117, 12, T6963C_WHITE);
  T6963C_Rectangle_Round_Edges(30, 30, 209, 97, 12, T6963C_WHITE);
  T6963C_Rectangle_Round_Edges(50, 50, 189, 77, 12, T6963C_WHITE);

  /*
   * Draw filled rounded edge rectangle
   */
  T6963C_Rectangle_Round_Edges_Fill(10, 10, 229, 117, 12, T6963C_WHITE);
  T6963C_Rectangle_Round_Edges_Fill(20, 20, 219, 107, 12, T6963C_BLACK);
  T6963C_Rectangle_Round_Edges_Fill(30, 30, 209, 97, 12, T6963C_WHITE);
  T6963C_Rectangle_Round_Edges_Fill(40, 40, 199, 87, 12, T6963C_BLACK);
  T6963C_Rectangle_Round_Edges_Fill(50, 50, 189, 77, 12, T6963C_WHITE);

  /*
   * Draw a cross
   */
  T6963C_line(0, 0, 239, 127, T6963C_WHITE);
  T6963C_line(0, 127, 239, 0, T6963C_WHITE);

  /*
   * Draw circles
   */
  T6963C_Circle(120, 64, 10, T6963C_WHITE);
  T6963C_Circle(120, 64, 30, T6963C_WHITE);
  T6963C_Circle(120, 64, 50, T6963C_WHITE);
  T6963C_Circle(120, 64, 70, T6963C_WHITE);
  T6963C_Circle(120, 64, 90, T6963C_WHITE);
  T6963C_Circle(120, 64, 110, T6963C_WHITE);
  T6963C_Circle(120, 64, 130, T6963C_WHITE);

  /*
   * Draw filled circles
   */
  T6963C_Circle_Fill(120, 64, 60, T6963C_WHITE);
  T6963C_Circle_Fill(120, 64, 55, T6963C_BLACK);
  T6963C_Circle_Fill(120, 64, 50, T6963C_WHITE);
  T6963C_Circle_Fill(120, 64, 45, T6963C_BLACK);
  T6963C_Circle_Fill(120, 64, 40, T6963C_WHITE);
  T6963C_Circle_Fill(120, 64, 35, T6963C_BLACK);
  T6963C_Circle_Fill(120, 64, 30, T6963C_WHITE);
  T6963C_Circle_Fill(120, 64, 25, T6963C_BLACK);
  T6963C_Circle_Fill(120, 64, 20, T6963C_WHITE);
  T6963C_Circle_Fill(120, 64, 15, T6963C_BLACK);
  T6963C_Circle_Fill(120, 64, 10, T6963C_WHITE);
  T6963C_Circle_Fill(120, 64, 5, T6963C_BLACK);

  Delay_ms(1000);
  T6963C_sprite(76, 4, einstein, 88, 119);         // Draw a sprite
  Delay_ms(1000);
 
  T6963C_SetGrPanel(1);                            // Select other graphic panel
  T6963C_image(mikroE_240x128_bmp);
  T6963C_DisplayGrPanel(1);
  Delay_ms(1000);
  T6963C_GrFill(0);
  T6963C_PartialImage(0, 0, 64, 64, 240, 128, mikroE_240x128_bmp);  // Display partial image
  Delay_ms(1000);
  T6963C_Graphics(0);
  T6963C_image(mikroE_240x128_bmp);
  T6963C_Graphics(1);
  T6963C_DisplayGrPanel(0);

  for(;;) {                                          // Endless loop
 
    /*
     * If RB0 is pressed, display only graphic panel
     */
    if(RB0_bit) {
      T6963C_Graphics(1);
      T6963C_Text(0);
      Delay_ms(300);
    }

    /*
     * If RB1 is pressed, toggle the display between graphic panel 0 and graphic panel 1
     */
    else if(RB1_bit) {
      panel++;
      panel &= 1;
      T6963C_DisplayGrPanel(panel);
      Delay_ms(300);
    }

    /*
     * If RB2 is pressed, display only text panel
     */
    else if(RB2_bit) {
      T6963C_Graphics(0);
      T6963C_Text(1);
      Delay_ms(300);
    }

    /*
     * If RB3 is pressed, display text and graphic panels
     */
    else if(RB3_bit) {
      T6963C_Graphics(1);
      T6963C_Text(1);
      Delay_ms(300);
    }
    
    /*
     * If RB4 is pressed, change cursor
     */
    else if(RB4_bit) {
      curs++;
      if(curs == 3) curs = 0;
      switch(curs) {
        case 0:
          // no cursor
          T6963C_Cursor(0);
          break;
        case 1:
          // blinking cursor
          T6963C_Cursor(1);
          T6963C_Cursor_Blink(1);
          break;
        case 2:
          // non blinking cursor
          T6963C_Cursor(1);
          T6963C_Cursor_Blink(0);
          break;
        }
      Delay_ms(300);
      }

    /*
     * If RB5 is pressed, perform the "Partial image" demostration
     */
     else if(RB5_bit) {
      T6963C_SetGrPanel(0);
      T6963C_SetTxtPanel(0);
      T6963C_TxtFill(0);
      T6963C_SetGrPanel(1);
      T6963C_SetTxtPanel(0);
      T6963C_Graphics(1);
      T6963C_Text(1);
      T6963C_DisplayGrPanel(1);
       T6963C_write_text(txt2, 5, 15, T6963C_ROM_MODE_XOR);
       Delay_1sec();
       T6963C_GrFill(0);
       T6963C_PartialImage(0, 0, 64, 64, 240, 128, mikroE_240x128_bmp);
       Delay_1sec();
       T6963C_PartialImage(0, 0, 128, 128, 240, 128, mikroE_240x128_bmp);
       Delay_1sec();
       T6963C_PartialImage(0, 0, 240, 128, 240, 128, mikroE_240x128_bmp);
       Delay_1sec();
      T6963C_TxtFill(0);
       T6963C_write_text(txt, 0, 0, T6963C_ROM_MODE_XOR);
       T6963C_write_text(txt1, 0, 15, T6963C_ROM_MODE_XOR);
    }

    /*
     * Move cursor, even if not visible
     */
    cposx++;
    if(cposx == T6963C_txtCols) {
      cposx = 0;
      cposy++;
      if(cposy == T6963C_grHeight / T6963C_CHARACTER_HEIGHT) {
        cposy = 0;
        }
      }
    T6963C_set_cursor(cposx, cposy);

    Delay_ms(100);
    }
}